@@ -72,8 +72,6 @@
  *	m4_dnl	  ACCESS_RTE;				this instruction needs route
  *	m4_dnl	  ACCESS_EATTRS;			this instruction needs extended attributes
  *
- *	m4_dnl	  METHOD_CONSTRUCTOR(name);		this instruction is in fact a method of the first argument's type; register it with the given name for that type
- *
  *	m4_dnl	  FID_MEMBER(				custom instruction member
  *	m4_dnl	    C type,				for storage in structs
  *	m4_dnl	    name,				how the member is named
@@ -101,11 +99,6 @@
  *
  *	Other code is just copied into the interpreter part.
  *
- *	It's also possible to declare type methods in a short way:
- *
- *	m4_dnl	METHOD(type, method name, argument count, code)
- *	m4_dnl	METHOD_R(type, method name, argument count, result type, union-field, value)
- *
  *	The filter language uses a simple type system, where values have types
  *	(constants T_*) and also terms (instructions) are statically typed. Our
  *	static typing is partial (some terms do not declare types of arguments
@@ -182,11 +175,11 @@
  *	m4_dnl	This structure is returned from the linearizer (105).
  *	m4_dnl	For writing directly to this structure, use FID_LINE_IN.
  *
- *	m4_dnl		f_dump_line_item_FI_EXAMPLE(struct dump_request *dreq, const struct f_line_item *item, const int indent)
+ *	m4_dnl		f_dump_line_item_FI_EXAMPLE(const struct f_line_item *item, const int indent)
  *	m4_dnl		{
  *	m4_dnl	(104)	  [[ put it here ]]
  *	m4_dnl		}
- *	m4_dnl	This code dumps the instruction via RDUMP. Note that the argument
+ *	m4_dnl	This code dumps the instruction on debug. Note that the argument
  *	m4_dnl	is the linearized instruction; if the instruction has arguments,
  *	m4_dnl	their code has already been linearized and their value is taken
  *	m4_dnl	from the value stack.
@@ -493,13 +486,26 @@
 
   INST(FI_DEFINED, 1, 1) {
     ARG_ANY(1);
-    RESULT(T_BOOL, i, (v1.type != T_VOID) && !val_is_undefined(v1));
+    RESULT(T_BOOL, i, (v1.type != T_VOID) && !undef_value(v1));
   }
 
-  METHOD_R(T_NET, type, T_ENUM_NET_TYPE, i, v1.val.net->type);
-  METHOD_R(T_IP, is_v4, T_BOOL, i, ipa_is_ip4(v1.val.ip));
+  INST(FI_TYPE, 1, 1) {
+    ARG_ANY(1); /* There may be more types supporting this operation */
+    switch (v1.type)
+    {
+      case T_NET:
+	RESULT(T_ENUM_NETTYPE, i, v1.val.net->type);
+	break;
+      default:
+	runtime( "Can't determine type of this item" );
+    }
+  }
+
+  INST(FI_IS_V4, 1, 1) {
+    ARG(1, T_IP);
+    RESULT(T_BOOL, i, ipa_is_ip4(v1.val.ip));
+  }
 
-  /* Add initialized variable */
   INST(FI_VAR_INIT, 1, 0) {
     NEVER_CONSTANT;
     ARG_ANY(1);
@@ -512,17 +518,6 @@
     fstk->vcnt = pos + 1;
   }
 
-  /* Add uninitialized variable */
-  INST(FI_VAR_INIT0, 0, 0) {
-    NEVER_CONSTANT;
-    SYMBOL;
-
-    /* New variable is always the last on stack */
-    uint pos = curline.vbase + sym->offset;
-    fstk->vstk[pos] = val_empty(sym->class & 0xff);
-    fstk->vcnt = pos + 1;
-  }
-
   /* Set to indirect value prepared in v1 */
   INST(FI_VAR_SET, 1, 0) {
     NEVER_CONSTANT;
@@ -553,84 +548,92 @@
     RESULT_VAL(val);
   }
 
-  METHOD_R(T_PATH, empty, T_PATH, ad, &null_adata);
-  METHOD_R(T_CLIST, empty, T_CLIST, ad, &null_adata);
-  METHOD_R(T_ECLIST, empty, T_ECLIST, ad, &null_adata);
-  METHOD_R(T_LCLIST, empty, T_LCLIST, ad, &null_adata);
+  INST(FI_FOR_INIT, 1, 0) {
+    NEVER_CONSTANT;
+    ARG_ANY(1);
+    SYMBOL;
 
-  /* Common loop begin instruction, always created by f_for_cycle() */
-  INST(FI_FOR_LOOP_START, 0, 3) {
+    FID_NEW_BODY()
+    ASSERT((sym->class & ~0xff) == SYM_VARIABLE);
+
+    /* Static type check */
+    if (f1->type)
+    {
+      enum f_type t_var = (sym->class & 0xff);
+      enum f_type t_arg = f_type_element_type(f1->type);
+      if (!t_arg)
+        cf_error("Value of expression in FOR must be iterable, got %s",
+		 f_type_name(f1->type));
+      if (t_var != t_arg)
+	cf_error("Loop variable '%s' in FOR must be %s, is %s",
+		 sym->name, f_type_name(t_arg), f_type_name(t_var));
+    }
+
+    FID_INTERPRET_BODY()
+
+    /* Dynamic type check */
+    if ((sym->class & 0xff) != f_type_element_type(v1.type))
+      runtime("Mismatched argument and variable type");
+
+    /* Setup the index */
+    v2 = (struct f_val) { .type = T_INT, .val.i = 0 };
+
+    /* Keep v1 and v2 on the stack */
+    fstk->vcnt += 2;
+  }
+
+  INST(FI_FOR_NEXT, 2, 0) {
     NEVER_CONSTANT;
     SYMBOL;
 
-    /* Repeat the instruction which called us */
-    ASSERT_DIE(fstk->ecnt > 1);
-    prevline.pos--;
+    /* Type checks are done in FI_FOR_INIT */
 
-    /* There should be exactly three items on the value stack to be taken care of */
-    fstk->vcnt += 3;
+    /* Loop variable */
+    struct f_val *var = &fstk->vstk[curline.vbase + sym->offset];
+    int step = 0;
 
-    /* And these should also stay there after we finish for the caller instruction */
-    curline.ventry += 3;
-
-    /* Assert the iterator variable positioning */
-    ASSERT_DIE(curline.vbase + sym->offset == fstk->vcnt - 1);
-
-    /* The result type declaration makes no sense here but is needed */
-    RESULT_TYPE(T_VOID);
-  }
-
-  /* Type-specific for_next iterators */
-  INST(FI_PATH_FOR_NEXT, 3, 0) {
-    NEVER_CONSTANT;
-    ARG(1, T_PATH);
-    if (as_path_walk(v1.val.ad, &v2.val.i, &v3.val.i))
-      LINE(2,0);
-
-    METHOD_CONSTRUCTOR("!for_next");
-  }
-
-  INST(FI_CLIST_FOR_NEXT, 3, 0) {
-    NEVER_CONSTANT;
-    ARG(1, T_CLIST);
-    if (int_set_walk(v1.val.ad, &v2.val.i, &v3.val.i))
-      LINE(2,0);
-
-    METHOD_CONSTRUCTOR("!for_next");
-  }
-
-  INST(FI_ECLIST_FOR_NEXT, 3, 0) {
-    NEVER_CONSTANT;
-    ARG(1, T_ECLIST);
-    if (ec_set_walk(v1.val.ad, &v2.val.i, &v3.val.ec))
-      LINE(2,0);
-
-    METHOD_CONSTRUCTOR("!for_next");
-  }
-
-  INST(FI_LCLIST_FOR_NEXT, 3, 0) {
-    NEVER_CONSTANT;
-    ARG(1, T_LCLIST);
-    if (lc_set_walk(v1.val.ad, &v2.val.i, &v3.val.lc))
-      LINE(2,0);
-
-    METHOD_CONSTRUCTOR("!for_next");
-  }
-
-  INST(FI_ROUTES_BLOCK_FOR_NEXT, 3, 0) {
-    NEVER_CONSTANT;
-    ARG(1, T_ROUTES_BLOCK);
-    if (!v2.type)
-      v2 = v1;
-
-    if (v2.val.rte)
+    switch(v1.type)
     {
-      v3.val.rte = v2.val.rte;
-      v2.val.rte = v2.val.rte->next;
-      LINE(2,0);
+    case T_PATH:
+      var->type = T_INT;
+      step = as_path_walk(v1.val.ad, &v2.val.i, &var->val.i);
+      break;
+
+    case T_CLIST:
+      var->type = T_PAIR;
+      step = int_set_walk(v1.val.ad, &v2.val.i, &var->val.i);
+      break;
+
+    case T_ECLIST:
+      var->type = T_EC;
+      step = ec_set_walk(v1.val.ad, &v2.val.i, &var->val.ec);
+      break;
+
+    case T_LCLIST:
+      var->type = T_LC;
+      step = lc_set_walk(v1.val.ad, &v2.val.i, &var->val.lc);
+      break;
+
+    default:
+      runtime( "Clist or lclist expected" );
     }
 
-    METHOD_CONSTRUCTOR("!for_next");
+    if (step)
+    {
+      /* Keep v1 and v2 on the stack */
+      fstk->vcnt += 2;
+
+      /* Repeat this instruction */
+      curline.pos--;
+
+      /* Execute the loop body */
+      LINE(1, 0);
+
+      /* Space for loop variable, may be unused */
+      fstk->vcnt += 1;
+    }
+    else
+      var->type = T_VOID;
   }
 
   INST(FI_CONDITION, 1, 0) {
@@ -641,12 +644,13 @@
       LINE(3,0);
   }
 
-  INST(FI_PRINT, 1, 0) {
+  INST(FI_PRINT, 0, 0) {
     NEVER_CONSTANT;
-    ARG_ANY(1);
+    VARARG;
 
-    if (!(fs->flags & FF_SILENT))
-      val_format(&v1, &fs->buf);
+    if (whati->varcount && !(fs->flags & FF_SILENT))
+      for (uint i=0; i<whati->varcount; i++)
+	val_format(&(vv(i)), &fs->buf);
   }
 
   INST(FI_FLUSH, 0, 0) {
@@ -670,13 +674,11 @@
     }
   }
 
-  INST(FI_RTA_GET, 1, 1) {
+  INST(FI_RTA_GET, 0, 1) {
     {
-      ACCESS_RTE;
-      ARG(1, T_ROUTE);
       STATIC_ATTR;
-
-      struct rta *rta = v1.val.rte ? v1.val.rte->attrs : (*fs->rte)->attrs;
+      ACCESS_RTE;
+      struct rta *rta = (*fs->rte)->attrs;
 
       switch (sa.sa_code)
       {
@@ -692,16 +694,6 @@
       case SA_WEIGHT:	RESULT(sa.f_type, i, rta->nh.weight + 1); break;
       case SA_PREF:	RESULT(sa.f_type, i, rta->pref); break;
       case SA_GW_MPLS:	RESULT(sa.f_type, i, rta->nh.labels ? rta->nh.label[0] : MPLS_NULL); break;
-      case SA_GW_MPLS_STACK:
-        {
-	  uint len = rta->nh.labels * sizeof(u32);
-	  struct adata *list = falloc(sizeof(struct adata) + len);
-	  list->length = len;
-	  memcpy(list->data, rta->nh.label, len);
-	  RESULT(sa.f_type, ad, list);
-	  break;
-	}
-
       case SA_ONLINK:	RESULT(sa.f_type, i, rta->nh.flags & RNF_ONLINK ? 1 : 0); break;
 
       default:
@@ -789,36 +781,6 @@
 	  }
 	  else
 	    rta->nh.labels = 0;
-
-	  rta->nh.labels_orig = rta->hostentry ? rta->nh.labels : 0;
-	}
-	break;
-
-      case SA_GW_MPLS_STACK:
-	{
-	  int len = int_set_get_size(v1.val.ad);
-	  u32 *l = int_set_get_data(v1.val.ad);
-
-	  if (len > MPLS_MAX_LABEL_STACK)
-	    runtime("Too many MPLS labels in stack (%d)", len);
-
-	  int i;
-	  for (i = 0; i < len; i++)
-	  {
-	    u32 label = l[i];
-
-	    if (label >= 0x100000)
-	      runtime("Invalid MPLS label (%u)", label);
-
-	    /* Ignore rest of label stack if implicit-NULL label (3) is set */
-	    if (label == MPLS_NULL)
-	      break;
-
-	    rta->nh.label[i] = label;
-	  }
-
-	  rta->nh.labels = i;
-	  rta->nh.labels_orig = rta->hostentry ? i : 0;
 	}
 	break;
 
@@ -855,18 +817,41 @@
     }
   }
 
-  INST(FI_EA_GET, 1, 1) {	/* Access to extended attributes */
+  INST(FI_EA_GET, 0, 1) {	/* Access to extended attributes */
+    DYNAMIC_ATTR;
     ACCESS_RTE;
     ACCESS_EATTRS;
-    ARG(1, T_ROUTE);
-    DYNAMIC_ATTR;
     RESULT_TYPE(da.f_type);
     {
-      struct ea_list *eal = v1.val.rte ? v1.val.rte->attrs->eattrs : *fs->eattrs;
-      eattr *e = ea_find(eal, da.ea_code);
+      eattr *e = ea_find(*fs->eattrs, da.ea_code);
 
       if (!e) {
-	RESULT_VAL(val_empty(da.f_type));
+	/* A special case: undefined as_path looks like empty as_path */
+	if (da.type == EAF_TYPE_AS_PATH) {
+	  RESULT_(T_PATH, ad, &null_adata);
+	  break;
+	}
+
+	/* The same special case for int_set */
+	if (da.type == EAF_TYPE_INT_SET) {
+	  RESULT_(T_CLIST, ad, &null_adata);
+	  break;
+	}
+
+	/* The same special case for ec_set */
+	if (da.type == EAF_TYPE_EC_SET) {
+	  RESULT_(T_ECLIST, ad, &null_adata);
+	  break;
+	}
+
+	/* The same special case for lc_set */
+	if (da.type == EAF_TYPE_LC_SET) {
+	  RESULT_(T_LCLIST, ad, &null_adata);
+	  break;
+	}
+
+	/* Undefined value */
+	RESULT_VOID;
 	break;
       }
 
@@ -878,10 +863,7 @@
 	RESULT_(T_QUAD, i, e->u.data);
 	break;
       case EAF_TYPE_OPAQUE:
-	if (da.f_type == T_ENUM_EMPTY)
-	  RESULT_(T_ENUM_EMPTY, i, 0);
-	else
-	  RESULT_(T_BYTESTRING, ad, e->u.ptr);
+	RESULT_(T_ENUM_EMPTY, i, 0);
 	break;
       case EAF_TYPE_IP_ADDRESS:
 	RESULT_(T_IP, ip, *((ip_addr *) e->u.ptr->data));
@@ -901,9 +883,6 @@
       case EAF_TYPE_LC_SET:
 	RESULT_(T_LCLIST, ad, e->u.ptr);
 	break;
-      case EAF_TYPE_STRING:
-	RESULT_(T_STRING, s, (const char *) e->u.ptr->data);
-	break;
       default:
 	bug("Unknown dynamic attribute type");
       }
@@ -916,12 +895,6 @@
     ARG_ANY(1);
     DYNAMIC_ATTR;
     ARG_TYPE(1, da.f_type);
-
-    FID_NEW_BODY;
-      if (da.f_type == T_ENUM_EMPTY)
-	cf_error("Setting opaque attribute is not allowed");
-
-    FID_INTERPRET_BODY;
     {
       struct ea_list *l = lp_alloc(fs->pool, sizeof(struct ea_list) + sizeof(eattr));
 
@@ -929,7 +902,7 @@
       l->flags = EALF_SORTED;
       l->count = 1;
       l->attrs[0].id = da.ea_code;
-      l->attrs[0].flags = da.flags;
+      l->attrs[0].flags = 0;
       l->attrs[0].type = da.type;
       l->attrs[0].originated = 1;
       l->attrs[0].fresh = 1;
@@ -941,6 +914,10 @@
 	l->attrs[0].u.data = v1.val.i;
 	break;
 
+      case EAF_TYPE_OPAQUE:
+	runtime( "Setting opaque attribute is not allowed" );
+	break;
+
       case EAF_TYPE_IP_ADDRESS:;
 	int len = sizeof(ip_addr);
 	struct adata *ad = lp_alloc(fs->pool, sizeof(struct adata) + len);
@@ -949,7 +926,6 @@
 	l->attrs[0].u.ptr = ad;
 	break;
 
-      case EAF_TYPE_OPAQUE:
       case EAF_TYPE_AS_PATH:
       case EAF_TYPE_INT_SET:
       case EAF_TYPE_EC_SET:
@@ -957,12 +933,6 @@
 	l->attrs[0].u.ptr = v1.val.ad;
 	break;
 
-      case EAF_TYPE_STRING:;
-	struct adata *d = lp_alloc_adata(fs->pool, strlen(v1.val.s) + 1);
-	memcpy(d->data, v1.val.s, d->length);
-	l->attrs[0].u.ptr = d;
-	break;
-
       case EAF_TYPE_BITFIELD:
 	{
 	  /* First, we have to find the old value */
@@ -995,16 +965,20 @@
     ea_unset_attr(fs->eattrs, fs->pool, 1, da.ea_code);
   }
 
-  /* Get length of */
-  METHOD_R(T_NET, len, T_INT, i, net_pxlen(v1.val.net));
-  METHOD_R(T_PATH, len, T_INT, i, as_path_getlen(v1.val.ad));
-  METHOD_R(T_CLIST, len, T_INT, i, int_set_get_size(v1.val.ad));
-  METHOD_R(T_ECLIST, len, T_INT, i, ec_set_get_size(v1.val.ad));
-  METHOD_R(T_LCLIST, len, T_INT, i, lc_set_get_size(v1.val.ad));
+  INST(FI_LENGTH, 1, 1) {	/* Get length of */
+    ARG_ANY(1);
+    switch(v1.type) {
+    case T_NET:    RESULT(T_INT, i, net_pxlen(v1.val.net)); break;
+    case T_PATH:   RESULT(T_INT, i, as_path_getlen(v1.val.ad)); break;
+    case T_CLIST:  RESULT(T_INT, i, int_set_get_size(v1.val.ad)); break;
+    case T_ECLIST: RESULT(T_INT, i, ec_set_get_size(v1.val.ad)); break;
+    case T_LCLIST: RESULT(T_INT, i, lc_set_get_size(v1.val.ad)); break;
+    default: runtime( "Prefix, path, clist or eclist expected" );
+    }
+  }
 
   INST(FI_NET_SRC, 1, 1) { 	/* Get src prefix */
     ARG(1, T_NET);
-    METHOD_CONSTRUCTOR("src");
 
     net_addr_union *net = (void *) v1.val.net;
     net_addr *src = falloc(sizeof(net_addr_ip6));
@@ -1040,7 +1014,6 @@
 
   INST(FI_NET_DST, 1, 1) { 	/* Get dst prefix */
     ARG(1, T_NET);
-    METHOD_CONSTRUCTOR("dst");
 
     net_addr_union *net = (void *) v1.val.net;
     net_addr *dst = falloc(sizeof(net_addr_ip6));
@@ -1074,78 +1047,156 @@
     RESULT(T_NET, net, dst);
   }
 
-  /* Get ROA max prefix length */
-  METHOD(T_NET, maxlen, 0, [[
+  INST(FI_ROA_MAXLEN, 1, 1) { 	/* Get ROA max prefix length */
+    ARG(1, T_NET);
     if (!net_is_roa(v1.val.net))
       runtime( "ROA expected" );
 
     RESULT(T_INT, i, (v1.val.net->type == NET_ROA4) ?
       ((net_addr_roa4 *) v1.val.net)->max_pxlen :
       ((net_addr_roa6 *) v1.val.net)->max_pxlen);
-  ]]);
+  }
 
-  /* Get ROA ASN */
-  METHOD(T_NET, asn, 0, [[
+  INST(FI_ASN, 1, 1) { 	/* Get ROA ASN or community ASN part */
+    ARG_ANY(1);
+    RESULT_TYPE(T_INT);
+    switch(v1.type)
+    {
+      case T_NET:
         if (!net_is_roa(v1.val.net))
           runtime( "ROA expected" );
 
-        RESULT(T_INT, i, (v1.val.net->type == NET_ROA4) ?
+        RESULT_(T_INT, i, (v1.val.net->type == NET_ROA4) ?
           ((net_addr_roa4 *) v1.val.net)->asn :
           ((net_addr_roa6 *) v1.val.net)->asn);
-  ]]);
+        break;
 
-  /* Convert prefix to IP */
-  METHOD_R(T_NET, ip, T_IP, ip, net_prefix(v1.val.net));
+      case T_PAIR:
+        RESULT_(T_INT, i, v1.val.i >> 16);
+        break;
+
+      case T_LC:
+        RESULT_(T_INT, i, v1.val.lc.asn);
+        break;
+
+      default:
+        runtime( "Net, pair or lc expected" );
+    }
+  }
+
+  INST(FI_IP, 1, 1) {	/* Convert prefix to ... */
+    ARG(1, T_NET);
+    RESULT(T_IP, ip, net_prefix(v1.val.net));
+  }
 
   INST(FI_ROUTE_DISTINGUISHER, 1, 1) {
     ARG(1, T_NET);
-    METHOD_CONSTRUCTOR("rd");
     if (!net_is_vpn(v1.val.net))
       runtime( "VPN address expected" );
     RESULT(T_RD, ec, net_rd(v1.val.net));
   }
 
-  /* Get first ASN from AS PATH */
-  METHOD_R(T_PATH, first, T_INT, i, ({ u32 as = 0; as_path_get_first(v1.val.ad, &as); as; }));
+  INST(FI_AS_PATH_FIRST, 1, 1) {	/* Get first ASN from AS PATH */
+    ARG(1, T_PATH);
+    u32 as = 0;
+    as_path_get_first(v1.val.ad, &as);
+    RESULT(T_INT, i, as);
+  }
 
-  /* Get last ASN from AS PATH */
-  METHOD_R(T_PATH, last, T_INT, i, ({ u32 as = 0; as_path_get_last(v1.val.ad, &as); as; }));
+  INST(FI_AS_PATH_LAST, 1, 1) {		/* Get last ASN from AS PATH */
+    ARG(1, T_PATH);
+    u32 as = 0;
+    as_path_get_last(v1.val.ad, &as);
+    RESULT(T_INT, i, as);
+  }
 
-  /* Get last ASN from non-aggregated part of AS PATH */
-  METHOD_R(T_PATH, last_nonaggregated, T_INT, i, as_path_get_last_nonaggregated(v1.val.ad));
+  INST(FI_AS_PATH_LAST_NAG, 1, 1) {	/* Get last ASN from non-aggregated part of AS PATH */
+    ARG(1, T_PATH);
+    RESULT(T_INT, i, as_path_get_last_nonaggregated(v1.val.ad));
+  }
 
-  /* Get ASN part from the standard community ASN */
-  METHOD_R(T_PAIR, asn, T_INT, i, v1.val.i >> 16);
+  INST(FI_PAIR_DATA, 1, 1) {	/* Get data part from the standard community */
+    ARG(1, T_PAIR);
+    RESULT(T_INT, i, v1.val.i & 0xFFFF);
+  }
 
-  /* Get data part from the standard community */
-  METHOD_R(T_PAIR, data, T_INT, i, v1.val.i & 0xFFFF);
+  INST(FI_LC_DATA1, 1, 1) {	/* Get data1 part from the large community */
+    ARG(1, T_LC);
+    RESULT(T_INT, i, v1.val.lc.ldp1);
+  }
 
-  /* Get ASN part from the large community */
-  METHOD_R(T_LC, asn, T_INT, i, v1.val.lc.asn);
+  INST(FI_LC_DATA2, 1, 1) {	/* Get data2 part from the large community */
+    ARG(1, T_LC);
+    RESULT(T_INT, i, v1.val.lc.ldp2);
+  }
 
-  /* Get data1 part from the large community */
-  METHOD_R(T_LC, data1, T_INT, i, v1.val.lc.ldp1);
+  INST(FI_MIN, 1, 1) {	/* Get minimum element from list */
+    ARG_ANY(1);
+    RESULT_TYPE(f_type_element_type(v1.type));
+    switch(v1.type)
+    {
+      case T_CLIST:
+        {
+          u32 val = 0;
+          int_set_min(v1.val.ad, &val);
+          RESULT_(T_PAIR, i, val);
+        }
+        break;
 
-  /* Get data2 part from the large community */
-  METHOD_R(T_LC, data2, T_INT, i, v1.val.lc.ldp2);
+      case T_ECLIST:
+        {
+          u64 val = 0;
+          ec_set_min(v1.val.ad, &val);
+          RESULT_(T_EC, ec, val);
+        }
+        break;
 
-  /* Get minimum element from clist */
-  METHOD_R(T_CLIST, min, T_PAIR, i, ({ u32 val = 0; int_set_min(v1.val.ad, &val); val; }));
+      case T_LCLIST:
+        {
+          lcomm val = { 0, 0, 0 };
+          lc_set_min(v1.val.ad, &val);
+          RESULT_(T_LC, lc, val);
+        }
+        break;
 
-  /* Get maximum element from clist */
-  METHOD_R(T_CLIST, max, T_PAIR, i, ({ u32 val = 0; int_set_max(v1.val.ad, &val); val; }));
+      default:
+        runtime( "Clist or lclist expected" );
+    }
+  }
 
-  /* Get minimum element from eclist */
-  METHOD_R(T_ECLIST, min, T_EC, ec, ({ u64 val = 0; ec_set_min(v1.val.ad, &val); val; }));
+  INST(FI_MAX, 1, 1) {	/* Get maximum element from list */
+    ARG_ANY(1);
+    RESULT_TYPE(f_type_element_type(v1.type));
+    switch(v1.type)
+    {
+      case T_CLIST:
+        {
+          u32 val = 0;
+          int_set_max(v1.val.ad, &val);
+          RESULT_(T_PAIR, i, val);
+        }
+        break;
 
-  /* Get maximum element from eclist */
-  METHOD_R(T_ECLIST, max, T_EC, ec, ({ u64 val = 0; ec_set_max(v1.val.ad, &val); val; }));
+      case T_ECLIST:
+        {
+          u64 val = 0;
+          ec_set_max(v1.val.ad, &val);
+          RESULT_(T_EC, ec, val);
+        }
+        break;
 
-  /* Get minimum element from lclist */
-  METHOD_R(T_LCLIST, min, T_LC, lc, ({ lcomm val = {}; lc_set_min(v1.val.ad, &val); val; }));
+      case T_LCLIST:
+        {
+          lcomm val = { 0, 0, 0 };
+          lc_set_max(v1.val.ad, &val);
+          RESULT_(T_LC, lc, val);
+        }
+        break;
 
-  /* Get maximum element from lclist */
-  METHOD_R(T_LCLIST, max, T_LC, lc, ({ lcomm val = {}; lc_set_max(v1.val.ad, &val); val; }));
+      default:
+        runtime( "Clist or lclist expected" );
+    }
+  }
 
   INST(FI_RETURN, 1, 0) {
     NEVER_CONSTANT;
@@ -1177,7 +1228,9 @@
     NEVER_CONSTANT;
     VARARG;
     SYMBOL;
-    RESULT_TYPE(sym->function->return_type);
+
+    /* Fake result type declaration */
+    RESULT_TYPE(T_VOID);
 
     FID_NEW_BODY()
     ASSERT(sym->class == SYM_FUNCTION);
@@ -1281,7 +1334,6 @@
   INST(FI_IP_MASK, 2, 1) { /* IP.MASK(val) */
     ARG(1, T_IP);
     ARG(2, T_INT);
-    METHOD_CONSTRUCTOR("mask");
     RESULT(T_IP, ip, [[ ipa_is_ip4(v1.val.ip) ?
       ipa_from_ip4(ip4_and(ipa_to_ip4(v1.val.ip), ip4_mkmask(v2.val.i))) :
       ipa_from_ip6(ip6_and(ipa_to_ip6(v1.val.ip), ip6_mkmask(v2.val.i))) ]]);
@@ -1290,268 +1342,165 @@
   INST(FI_PATH_PREPEND, 2, 1) {	/* Path prepend */
     ARG(1, T_PATH);
     ARG(2, T_INT);
-    METHOD_CONSTRUCTOR("prepend");
     RESULT(T_PATH, ad, [[ as_path_prepend(fpool, v1.val.ad, v2.val.i) ]]);
   }
 
-  /* Community list add */
-  INST(FI_CLIST_ADD_PAIR, 2, 1) {
-    ARG(1, T_CLIST);
-    ARG(2, T_PAIR);
-    METHOD_CONSTRUCTOR("add");
-    RESULT(T_CLIST, ad, [[ int_set_add(fpool, v1.val.ad, v2.val.i) ]]);
+  INST(FI_CLIST_ADD, 2, 1) {	/* (Extended) Community list add */
+    ARG_ANY(1);
+    ARG_ANY(2);
+    RESULT_TYPE(f1->type);
+
+    if (v1.type == T_PATH)
+      runtime("Can't add to path");
+
+    else if (v1.type == T_CLIST)
+    {
+      /* Community (or cluster) list */
+      struct f_val dummy;
+
+      if ((v2.type == T_PAIR) || (v2.type == T_QUAD))
+	RESULT_(T_CLIST, ad, [[ int_set_add(fpool, v1.val.ad, v2.val.i) ]]);
+      /* IP->Quad implicit conversion */
+      else if (val_is_ip4(&v2))
+	RESULT_(T_CLIST, ad, [[ int_set_add(fpool, v1.val.ad, ipa_to_u32(v2.val.ip)) ]]);
+      else if ((v2.type == T_SET) && clist_set_type(v2.val.t, &dummy))
+	runtime("Can't add set");
+      else if (v2.type == T_CLIST)
+	RESULT_(T_CLIST, ad, [[ int_set_union(fpool, v1.val.ad, v2.val.ad) ]]);
+      else
+	runtime("Can't add non-pair");
+    }
+
+    else if (v1.type == T_ECLIST)
+    {
+      /* v2.val is either EC or EC-set */
+      if ((v2.type == T_SET) && eclist_set_type(v2.val.t))
+	runtime("Can't add set");
+      else if (v2.type == T_ECLIST)
+	RESULT_(T_ECLIST, ad, [[ ec_set_union(fpool, v1.val.ad, v2.val.ad) ]]);
+      else if (v2.type != T_EC)
+	runtime("Can't add non-ec");
+      else
+	RESULT_(T_ECLIST, ad, [[ ec_set_add(fpool, v1.val.ad, v2.val.ec) ]]);
+    }
+
+    else if (v1.type == T_LCLIST)
+    {
+      /* v2.val is either LC or LC-set */
+      if ((v2.type == T_SET) && lclist_set_type(v2.val.t))
+	runtime("Can't add set");
+      else if (v2.type == T_LCLIST)
+	RESULT_(T_LCLIST, ad, [[ lc_set_union(fpool, v1.val.ad, v2.val.ad) ]]);
+      else if (v2.type != T_LC)
+	runtime("Can't add non-lc");
+      else
+	RESULT_(T_LCLIST, ad, [[ lc_set_add(fpool, v1.val.ad, v2.val.lc) ]]);
+
+    }
+
+    else
+      runtime("Can't add to non-[e|l]clist");
   }
 
-  /* Hack for gw_mpls_list */
-  INST(FI_CLIST_ADD_INT, 2, 1) {
-    ARG(1, T_CLIST);
-    ARG(2, T_INT);
-    METHOD_CONSTRUCTOR("add");
-    RESULT(T_CLIST, ad, [[ int_set_add(fpool, v1.val.ad, v2.val.i) ]]);
+  INST(FI_CLIST_DEL, 2, 1) {	/* (Extended) Community list add or delete */
+    ARG_ANY(1);
+    ARG_ANY(2);
+    RESULT_TYPE(f1->type);
+
+    if (v1.type == T_PATH)
+    {
+      if ((v2.type == T_SET) && path_set_type(v2.val.t) || (v2.type == T_INT))
+	RESULT_(T_PATH, ad, [[ as_path_filter(fpool, v1.val.ad, &v2, 0) ]]);
+      else
+	runtime("Can't delete non-integer (set)");
+    }
+
+    else if (v1.type == T_CLIST)
+    {
+      /* Community (or cluster) list */
+      struct f_val dummy;
+
+      if ((v2.type == T_PAIR) || (v2.type == T_QUAD))
+	RESULT_(T_CLIST, ad, [[ int_set_del(fpool, v1.val.ad, v2.val.i) ]]);
+      /* IP->Quad implicit conversion */
+      else if (val_is_ip4(&v2))
+	RESULT_(T_CLIST, ad, [[ int_set_del(fpool, v1.val.ad, ipa_to_u32(v2.val.ip)) ]]);
+      else if ((v2.type == T_SET) && clist_set_type(v2.val.t, &dummy) || (v2.type == T_CLIST))
+	RESULT_(T_CLIST, ad, [[ clist_filter(fpool, v1.val.ad, &v2, 0) ]]);
+      else
+	runtime("Can't delete non-pair");
+    }
+
+    else if (v1.type == T_ECLIST)
+    {
+      /* v2.val is either EC or EC-set */
+      if ((v2.type == T_SET) && eclist_set_type(v2.val.t) || (v2.type == T_ECLIST))
+	RESULT_(T_ECLIST, ad, [[ eclist_filter(fpool, v1.val.ad, &v2, 0) ]]);
+      else if (v2.type != T_EC)
+	runtime("Can't delete non-ec");
+      else
+	RESULT_(T_ECLIST, ad, [[ ec_set_del(fpool, v1.val.ad, v2.val.ec) ]]);
+    }
+
+    else if (v1.type == T_LCLIST)
+    {
+      /* v2.val is either LC or LC-set */
+      if ((v2.type == T_SET) && lclist_set_type(v2.val.t) || (v2.type == T_LCLIST))
+	RESULT_(T_LCLIST, ad, [[ lclist_filter(fpool, v1.val.ad, &v2, 0) ]]);
+      else if (v2.type != T_LC)
+	runtime("Can't delete non-lc");
+      else
+	RESULT_(T_LCLIST, ad, [[ lc_set_del(fpool, v1.val.ad, v2.val.lc) ]]);
+    }
+
+    else
+      runtime("Can't delete in non-[e|l]clist");
   }
 
-  INST(FI_CLIST_ADD_IP, 2, 1) {
-    ARG(1, T_CLIST);
-    ARG(2, T_IP);
-    METHOD_CONSTRUCTOR("add");
+  INST(FI_CLIST_FILTER, 2, 1) {	/* (Extended) Community list add or delete */
+    ARG_ANY(1);
+    ARG_ANY(2);
+    RESULT_TYPE(f1->type);
 
-    FID_NEW_BODY();
-    /* IP->Quad implicit conversion, must be before FI_CLIST_ADD_QUAD */
-    cf_warn("Method add(clist, ip) is deprecated, please use add(clist, quad)");
+    if (v1.type == T_PATH)
+    {
+      if ((v2.type == T_SET) && path_set_type(v2.val.t))
+	RESULT_(T_PATH, ad, [[ as_path_filter(fpool, v1.val.ad, &v2, 1) ]]);
+      else
+	runtime("Can't filter integer");
+    }
 
-    FID_INTERPRET_BODY();
-    if (!val_is_ip4(&v2)) runtime("Mismatched IP type");
-    RESULT(T_CLIST, ad, [[ int_set_add(fpool, v1.val.ad, ipa_to_u32(v2.val.ip)) ]]);
-  }
+    else if (v1.type == T_CLIST)
+    {
+      /* Community (or cluster) list */
+      struct f_val dummy;
 
-  INST(FI_CLIST_ADD_QUAD, 2, 1) {
-    ARG(1, T_CLIST);
-    ARG(2, T_QUAD);
-    METHOD_CONSTRUCTOR("add");
-    RESULT(T_CLIST, ad, [[ int_set_add(fpool, v1.val.ad, v2.val.i) ]]);
-  }
+      if ((v2.type == T_SET) && clist_set_type(v2.val.t, &dummy) || (v2.type == T_CLIST))
+	RESULT_(T_CLIST, ad, [[ clist_filter(fpool, v1.val.ad, &v2, 1) ]]);
+      else
+	runtime("Can't filter pair");
+    }
 
-  INST(FI_CLIST_ADD_CLIST, 2, 1) {
-    ARG(1, T_CLIST);
-    ARG(2, T_CLIST);
-    METHOD_CONSTRUCTOR("add");
-    RESULT(T_CLIST, ad, [[ int_set_union(fpool, v1.val.ad, v2.val.ad) ]]);
-  }
+    else if (v1.type == T_ECLIST)
+    {
+      /* v2.val is either EC or EC-set */
+      if ((v2.type == T_SET) && eclist_set_type(v2.val.t) || (v2.type == T_ECLIST))
+	RESULT_(T_ECLIST, ad, [[ eclist_filter(fpool, v1.val.ad, &v2, 1) ]]);
+      else
+	runtime("Can't filter ec");
+    }
 
-  INST(FI_ECLIST_ADD_EC, 2, 1) {
-    ARG(1, T_ECLIST);
-    ARG(2, T_EC);
-    METHOD_CONSTRUCTOR("add");
-    RESULT(T_ECLIST, ad, [[ ec_set_add(fpool, v1.val.ad, v2.val.ec) ]]);
-  }
+    else if (v1.type == T_LCLIST)
+    {
+      /* v2.val is either LC or LC-set */
+      if ((v2.type == T_SET) && lclist_set_type(v2.val.t) || (v2.type == T_LCLIST))
+	RESULT_(T_LCLIST, ad, [[ lclist_filter(fpool, v1.val.ad, &v2, 1) ]]);
+      else
+	runtime("Can't filter lc");
+    }
 
-  INST(FI_ECLIST_ADD_ECLIST, 2, 1) {
-    ARG(1, T_ECLIST);
-    ARG(2, T_ECLIST);
-    METHOD_CONSTRUCTOR("add");
-    RESULT(T_ECLIST, ad, [[ ec_set_union(fpool, v1.val.ad, v2.val.ad) ]]);
-  }
-
-  INST(FI_LCLIST_ADD_LC, 2, 1) {
-    ARG(1, T_LCLIST);
-    ARG(2, T_LC);
-    METHOD_CONSTRUCTOR("add");
-    RESULT(T_LCLIST, ad, [[ lc_set_add(fpool, v1.val.ad, v2.val.lc) ]]);
-  }
-
-  INST(FI_LCLIST_ADD_LCLIST, 2, 1) {
-    ARG(1, T_LCLIST);
-    ARG(2, T_LCLIST);
-    METHOD_CONSTRUCTOR("add");
-    RESULT(T_LCLIST, ad, [[ lc_set_union(fpool, v1.val.ad, v2.val.ad) ]]);
-  }
-
-  INST(FI_PATH_DELETE_INT, 2, 1) {
-    ARG(1, T_PATH);
-    ARG(2, T_INT);
-    METHOD_CONSTRUCTOR("delete");
-    RESULT(T_PATH, ad, [[ as_path_filter(fpool, v1.val.ad, &v2, 0) ]]);
-  }
-
-  INST(FI_PATH_DELETE_SET, 2, 1) {
-    ARG(1, T_PATH);
-    ARG(2, T_SET);
-    METHOD_CONSTRUCTOR("delete");
-
-    if (!path_set_type(v2.val.t))
-      runtime("Mismatched set type");
-
-    RESULT(T_PATH, ad, [[ as_path_filter(fpool, v1.val.ad, &v2, 0) ]]);
-  }
-
-  /* Community list delete */
-  INST(FI_CLIST_DELETE_PAIR, 2, 1) {
-    ARG(1, T_CLIST);
-    ARG(2, T_PAIR);
-    METHOD_CONSTRUCTOR("delete");
-    RESULT(T_CLIST, ad, [[ int_set_del(fpool, v1.val.ad, v2.val.i) ]]);
-  }
-
-  /* Hack for gw_mpls_list */
-  INST(FI_CLIST_DELETE_INT, 2, 1) {
-    ARG(1, T_CLIST);
-    ARG(2, T_INT);
-    METHOD_CONSTRUCTOR("delete");
-    RESULT(T_CLIST, ad, [[ int_set_del(fpool, v1.val.ad, v2.val.i) ]]);
-  }
-
-  INST(FI_CLIST_DELETE_IP, 2, 1) {
-    ARG(1, T_CLIST);
-    ARG(2, T_IP);
-    METHOD_CONSTRUCTOR("delete");
-
-    FID_NEW_BODY();
-    /* IP->Quad implicit conversion, must be before FI_CLIST_DELETE_QUAD */
-    cf_warn("Method delete(clist, ip) is deprecated, please use delete(clist, quad)");
-
-    FID_INTERPRET_BODY();
-    if (!val_is_ip4(&v2)) runtime("Mismatched IP type");
-    RESULT(T_CLIST, ad, [[ int_set_del(fpool, v1.val.ad, ipa_to_u32(v2.val.ip)) ]]);
-  }
-
-  INST(FI_CLIST_DELETE_QUAD, 2, 1) {
-    ARG(1, T_CLIST);
-    ARG(2, T_QUAD);
-    METHOD_CONSTRUCTOR("delete");
-    RESULT(T_CLIST, ad, [[ int_set_del(fpool, v1.val.ad, v2.val.i) ]]);
-  }
-
-  INST(FI_CLIST_DELETE_CLIST, 2, 1) {
-    ARG(1, T_CLIST);
-    ARG(2, T_CLIST);
-    METHOD_CONSTRUCTOR("delete");
-    RESULT(T_CLIST, ad, [[ clist_filter(fpool, v1.val.ad, &v2, 0) ]]);
-  }
-
-  INST(FI_CLIST_DELETE_SET, 2, 1) {
-    ARG(1, T_CLIST);
-    ARG(2, T_SET);
-    METHOD_CONSTRUCTOR("delete");
-
-    if (!clist_set_type(v2.val.t, &(struct f_val){}))
-      runtime("Mismatched set type");
-
-    RESULT(T_CLIST, ad, [[ clist_filter(fpool, v1.val.ad, &v2, 0) ]]);
-  }
-
-  INST(FI_ECLIST_DELETE_EC, 2, 1) {
-    ARG(1, T_ECLIST);
-    ARG(2, T_EC);
-    METHOD_CONSTRUCTOR("delete");
-    RESULT(T_ECLIST, ad, [[ ec_set_del(fpool, v1.val.ad, v2.val.ec) ]]);
-  }
-
-  INST(FI_ECLIST_DELETE_ECLIST, 2, 1) {
-    ARG(1, T_ECLIST);
-    ARG(2, T_ECLIST);
-    METHOD_CONSTRUCTOR("delete");
-    RESULT(T_ECLIST, ad, [[ eclist_filter(fpool, v1.val.ad, &v2, 0) ]]);
-  }
-
-  INST(FI_ECLIST_DELETE_SET, 2, 1) {
-    ARG(1, T_ECLIST);
-    ARG(2, T_SET);
-    METHOD_CONSTRUCTOR("delete");
-
-    if (!eclist_set_type(v2.val.t))
-      runtime("Mismatched set type");
-
-    RESULT(T_ECLIST, ad, [[ eclist_filter(fpool, v1.val.ad, &v2, 0) ]]);
-  }
-
-  INST(FI_LCLIST_DELETE_LC, 2, 1) {
-    ARG(1, T_LCLIST);
-    ARG(2, T_LC);
-    METHOD_CONSTRUCTOR("delete");
-    RESULT(T_LCLIST, ad, [[ lc_set_del(fpool, v1.val.ad, v2.val.lc) ]]);
-  }
-
-  INST(FI_LCLIST_DELETE_LCLIST, 2, 1) {
-    ARG(1, T_LCLIST);
-    ARG(2, T_LCLIST);
-    METHOD_CONSTRUCTOR("delete");
-    RESULT(T_LCLIST, ad, [[ lclist_filter(fpool, v1.val.ad, &v2, 0) ]]);
-  }
-
-  INST(FI_LCLIST_DELETE_SET, 2, 1) {
-    ARG(1, T_LCLIST);
-    ARG(2, T_SET);
-    METHOD_CONSTRUCTOR("delete");
-
-    if (!lclist_set_type(v2.val.t))
-      runtime("Mismatched set type");
-
-    RESULT(T_LCLIST, ad, [[ lclist_filter(fpool, v1.val.ad, &v2, 0) ]]);
-  }
-
-  INST(FI_PATH_FILTER_SET, 2, 1) {
-    ARG(1, T_PATH);
-    ARG(2, T_SET);
-    METHOD_CONSTRUCTOR("filter");
-
-    if (!path_set_type(v2.val.t))
-      runtime("Mismatched set type");
-
-    RESULT(T_PATH, ad, [[ as_path_filter(fpool, v1.val.ad, &v2, 1) ]]);
-  }
-
-  INST(FI_CLIST_FILTER_CLIST, 2, 1) {
-    ARG(1, T_CLIST);
-    ARG(2, T_CLIST);
-    METHOD_CONSTRUCTOR("filter");
-    RESULT(T_CLIST, ad, [[ clist_filter(fpool, v1.val.ad, &v2, 1) ]]);
-  }
-
-  INST(FI_CLIST_FILTER_SET, 2, 1) {
-    ARG(1, T_CLIST);
-    ARG(2, T_SET);
-    METHOD_CONSTRUCTOR("filter");
-
-    if (!clist_set_type(v2.val.t, &(struct f_val){}))
-      runtime("Mismatched set type");
-
-    RESULT(T_CLIST, ad, [[ clist_filter(fpool, v1.val.ad, &v2, 1) ]]);
-  }
-
-  INST(FI_ECLIST_FILTER_ECLIST, 2, 1) {
-    ARG(1, T_ECLIST);
-    ARG(2, T_ECLIST);
-    METHOD_CONSTRUCTOR("filter");
-    RESULT(T_ECLIST, ad, [[ eclist_filter(fpool, v1.val.ad, &v2, 1) ]]);
-  }
-
-  INST(FI_ECLIST_FILTER_SET, 2, 1) {
-    ARG(1, T_ECLIST);
-    ARG(2, T_SET);
-    METHOD_CONSTRUCTOR("filter");
-
-    if (!eclist_set_type(v2.val.t))
-      runtime("Mismatched set type");
-
-    RESULT(T_ECLIST, ad, [[ eclist_filter(fpool, v1.val.ad, &v2, 1) ]]);
-  }
-
-  INST(FI_LCLIST_FILTER_LCLIST, 2, 1) {
-    ARG(1, T_LCLIST);
-    ARG(2, T_LCLIST);
-    METHOD_CONSTRUCTOR("filter");
-    RESULT(T_LCLIST, ad, [[ lclist_filter(fpool, v1.val.ad, &v2, 1) ]]);
-  }
-
-  INST(FI_LCLIST_FILTER_SET, 2, 1) {
-    ARG(1, T_LCLIST);
-    ARG(2, T_SET);
-    METHOD_CONSTRUCTOR("filter");
-
-    if (!lclist_set_type(v2.val.t))
-      runtime("Mismatched set type");
-
-    RESULT(T_LCLIST, ad, [[ lclist_filter(fpool, v1.val.ad, &v2, 1) ]]);
+    else
+      runtime("Can't filter non-[e|l]clist");
   }
 
   INST(FI_ROA_CHECK_IMPLICIT, 0, 1) {	/* ROA Check */
@@ -1606,37 +1555,6 @@
 
   }
 
-  INST(FI_ASPA_CHECK_EXPLICIT, 2, 1) {	/* ASPA Check */
-    NEVER_CONSTANT;
-    ARG(1, T_PATH);
-    ARG(2, T_BOOL);
-    RTC(3);
-    struct rtable *table = rtc->table;
-
-    if (!table)
-      runtime("Missing ASPA table");
-
-    if (table->addr_type != NET_ASPA)
-      runtime("Table type must be ASPA");
-
-    RESULT(T_ENUM_ASPA, i, [[ aspa_check(table, v1.val.ad, v2.val.i) ]]);
-  }
-
-  INST(FI_FROM_HEX, 1, 1) {	/* Convert hex text to bytestring */
-    ARG(1, T_STRING);
-
-    int len = bstrhextobin(v1.val.s, NULL);
-    if (len < 0)
-      runtime("Invalid hex string");
-
-    struct adata *bs;
-    bs = falloc(sizeof(struct adata) + len);
-    bs->length = bstrhextobin(v1.val.s, bs->data);
-    ASSERT(bs->length == (size_t) len);
-
-    RESULT(T_BYTESTRING, bs, bs);
-  }
-
   INST(FI_FORMAT, 1, 1) {	/* Format */
     ARG_ANY(1);
     RESULT(T_STRING, s, val_format_str(fpool, &v1));
