@@ -31,7 +31,6 @@ struct channel;
 struct ea_list;
 struct eattr;
 struct symbol;
-struct mpls_fec_map;
 
 
 /*
@@ -40,7 +39,6 @@ struct mpls_fec_map;
 
 enum protocol_class {
   PROTOCOL_NONE,
-  PROTOCOL_AGGREGATOR,
   PROTOCOL_BABEL,
   PROTOCOL_BFD,
   PROTOCOL_BGP,
@@ -48,7 +46,6 @@ enum protocol_class {
   PROTOCOL_DEVICE,
   PROTOCOL_DIRECT,
   PROTOCOL_KERNEL,
-  PROTOCOL_L3VPN,
   PROTOCOL_OSPF,
   PROTOCOL_MRT,
   PROTOCOL_PERF,
@@ -77,7 +74,7 @@ struct protocol {
   void (*postconfig)(struct proto_config *);			/* After configuring each instance */
   struct proto * (*init)(struct proto_config *);		/* Create new instance */
   int (*reconfigure)(struct proto *, struct proto_config *);	/* Try to reconfigure instance, returns success */
-  void (*dump)(struct proto *, struct dump_request *);		/* Debugging dump */
+  void (*dump)(struct proto *);			/* Debugging dump */
   int (*start)(struct proto *);			/* Start the instance */
   int (*shutdown)(struct proto *);		/* Stop the instance */
   void (*cleanup)(struct proto *);		/* Called after shutdown when protocol became hungry/down */
@@ -93,7 +90,7 @@ void proto_build(struct protocol *);	/* Called from protocol to register itself
 void protos_preconfig(struct config *);
 void protos_commit(struct config *new, struct config *old, int force_restart, int type);
 struct proto * proto_spawn(struct proto_config *cf, uint disabled);
-void protos_dump_all(struct dump_request *);
+void protos_dump_all(void);
 
 #define GA_UNKNOWN	0		/* Attribute not recognized */
 #define GA_NAME		1		/* Result = name */
@@ -106,7 +103,7 @@ void protos_dump_all(struct dump_request *);
 
 extern struct protocol
   proto_device, proto_radv, proto_rip, proto_static, proto_mrt,
-  proto_ospf, proto_perf, proto_l3vpn, proto_aggregator,
+  proto_ospf, proto_perf,
   proto_pipe, proto_bgp, proto_bmp, proto_bfd, proto_babel, proto_rpki;
 
 /*
@@ -127,7 +124,6 @@ struct proto_config {
   u8 vrf_set;				/* Related VRF instance (below) is defined */
   u32 debug, mrtdump;			/* Debugging bitfields, both use D_* constants */
   u32 router_id;			/* Protocol specific router ID */
-  const char *hostname;			/* Protocol specific hostname */
 
   list channels;			/* List of channel configs (struct channel_config) */
   struct iface *vrf;			/* Related VRF instance, NULL if global */
@@ -175,8 +171,6 @@ struct proto {
   struct channel *main_channel;		/* Primary channel */
   struct rte_src *main_source;		/* Primary route source */
   struct iface *vrf;			/* Related VRF instance, NULL if global */
-  struct channel *mpls_channel;		/* MPLS channel, when used */
-  struct mpls_fec_map *mpls_map;	/* Maps protocol routes to FECs / labels */
 
   const char *name;				/* Name of this instance (== cf->name) */
   u32 debug;				/* Debugging flags */
@@ -220,6 +214,7 @@ struct proto {
   void (*if_notify)(struct proto *, unsigned flags, struct iface *i);
   void (*ifa_notify)(struct proto *, unsigned flags, struct ifa *a);
   void (*rt_notify)(struct proto *, struct channel *, struct network *net, struct rte *new, struct rte *old);
+  void (*rte_update_in_notify)(struct channel *, const net_addr *, const struct rte *, const struct rte_src *);
   void (*neigh_notify)(struct neighbor *neigh);
   int (*preexport)(struct channel *, struct rte *rt);
   void (*reload_routes)(struct channel *);
@@ -312,12 +307,6 @@ proto_get_router_id(struct proto_config *pc)
   return pc->router_id ? pc->router_id : pc->global->router_id;
 }
 
-static inline const char*
-proto_get_hostname(struct proto_config *pc)
-{
-  return pc->hostname ? pc->hostname : pc->global->hostname;
-}
-
 
 extern pool *proto_pool;
 extern list proto_list;
@@ -488,8 +477,7 @@ struct channel_class {
 #endif
 };
 
-extern const struct channel_class channel_basic;
-extern const struct channel_class channel_bgp;
+extern struct channel_class channel_bgp;
 
 struct channel_config {
   node n;
@@ -512,7 +500,6 @@ struct channel_config {
   u8 merge_limit;			/* Maximal number of nexthops for RA_MERGED */
   u8 in_keep_filtered;			/* Routes rejected in import filter are kept */
   u8 rpki_reload;			/* RPKI changes trigger channel reload */
-  u8 bmp_hack;				/* No flush */
 };
 
 struct channel {
@@ -565,7 +552,6 @@ struct channel {
   u8 reload_pending;			/* Reloading and another reload is scheduled */
   u8 refeed_pending;			/* Refeeding and another refeed is scheduled */
   u8 rpki_reload;			/* RPKI changes trigger channel reload */
-  u8 bmp_hack;				/* No flush */
 
   struct rtable *out_table;		/* Internal table for exported routes */
 
@@ -578,7 +564,7 @@ struct channel {
  *
  * CS_DOWN - The initial and the final state of a channel. There is no route
  * exchange between the protocol and the table. Channel is not counted as
- * active. Channel keeps a ptr to the table, keeps the table locked, but is
+ * active. Channel keeps a ptr to the table, but do not lock the table and is
  * not linked in the table. Generally, new closed channels are created in
  * protocols' init() hooks. The protocol is expected to explicitly activate its
  * channels (by calling channel_init() or channel_open()).
@@ -602,8 +588,8 @@ struct channel {
  * channel. The channel is still initialized, but no route exchange is allowed.
  * Instead, the associated table is running flush loop to remove routes imported
  * through the channel. After that, the channel changes state to CS_DOWN and
- * is detached from the table (the channel is unlinked from it, but keeps the
- * table locked). Unlike other states, the CS_FLUSHING state is not explicitly
+ * is detached from the table (the table is unlocked and the channel is unlinked
+ * from it). Unlike other states, the CS_FLUSHING state is not explicitly
  * entered or left by the protocol. A protocol may request to close a channel
  * (by calling channel_close()), which causes the channel to change state to
  * CS_FLUSHING and later to CS_DOWN. Also note that channels are closed
@@ -630,16 +616,11 @@ struct channel {
 struct channel_config *proto_cf_find_channel(struct proto_config *p, uint net_type);
 static inline struct channel_config *proto_cf_main_channel(struct proto_config *pc)
 { return proto_cf_find_channel(pc, pc->net_type); }
-static inline struct channel_config *proto_cf_mpls_channel(struct proto_config *pc)
-{ return (pc->net_type != NET_MPLS) ? proto_cf_find_channel(pc, NET_MPLS) : NULL; }
 
 struct channel *proto_find_channel_by_table(struct proto *p, struct rtable *t);
 struct channel *proto_find_channel_by_name(struct proto *p, const char *n);
 struct channel *proto_add_channel(struct proto *p, struct channel_config *cf);
-void proto_remove_channel(struct proto *p, struct channel *c);
 int proto_configure_channel(struct proto *p, struct channel **c, struct channel_config *cf);
-void proto_setup_mpls_map(struct proto *p, uint rts, int hooks);
-void proto_shutdown_mpls_map(struct proto *p, int hooks);
 
 void channel_set_state(struct channel *c, uint state);
 void channel_setup_in_table(struct channel *c);
