@@ -17,7 +17,6 @@
 #include "nest/protocol.h"
 #include "nest/route.h"
 #include "nest/attrs.h"
-#include "nest/mpls.h"
 #include "conf/conf.h"
 #include "lib/resource.h"
 #include "lib/string.h"
@@ -89,7 +88,7 @@ bgp_set_attr(ea_list **attrs, struct linpool *pool, uint code, uint flags, uintp
       attrs,
       pool,
       EA_CODE(PROTOCOL_BGP, code),
-      bgp_attr_table[code].flags | (flags & BAF_PARTIAL),
+      flags & ~BAF_EXT_LEN,
       bgp_attr_table[code].type,
       val
   );
@@ -690,7 +689,7 @@ static void
 bgp_format_cluster_list(const eattr *a, byte *buf, uint size)
 {
   /* Truncates cluster lists larger than buflen, probably not a problem */
-  int_set_format(a->u.ptr, ISF_ROUTER_ID, -1, buf, size);
+  int_set_format(a->u.ptr, 0, -1, buf, size);
 }
 
 
@@ -1152,13 +1151,14 @@ bgp_attr_known(uint code)
   return (code < ARRAY_SIZE(bgp_attr_table)) && bgp_attr_table[code].name;
 }
 
-const char *
-bgp_attr_name(uint code)
+void bgp_fix_attr_flags(ea_list *attrs)
 {
-  return (code < ARRAY_SIZE(bgp_attr_table)) ? bgp_attr_table[code].name : NULL;
+  for (u8 i = 0; i < attrs->count; i++)
+  {
+    attrs->attrs[i].flags = bgp_attr_table[EA_ID(attrs->attrs[i].id)].flags;
+  }
 }
 
-
 /*
  *	Attribute export
  */
@@ -1175,8 +1175,7 @@ bgp_export_attr(struct bgp_export_state *s, eattr *a, ea_list *to)
   {
     const struct bgp_attr_desc *desc = &bgp_attr_table[code];
 
-    /* The flags should be correct, we reset them just to be sure */
-    ASSERT(!((a->flags ^ desc->flags) & (BAF_OPTIONAL | BAF_TRANSITIVE)));
+    /* The flags might have been zero if the attr was added by filters */
     a->flags = (a->flags & BAF_PARTIAL) | desc->flags;
 
     /* Set partial bit if new opt-trans attribute is attached to non-local route */
@@ -1247,11 +1246,7 @@ bgp_export_attrs(struct bgp_export_state *s, ea_list *attrs)
 static inline int
 bgp_encode_attr(struct bgp_write_state *s, eattr *a, byte *buf, uint size)
 {
-  if (EA_PROTO(a->id) != PROTOCOL_BGP)
-    if (s->ignore_non_bgp_attrs)
-      return 0;
-    else
-      bug("Tried to encode a non-BGP attribute");
+  ASSERT(EA_PROTO(a->id) == PROTOCOL_BGP);
 
   uint code = EA_ID(a->id);
 
@@ -1506,13 +1501,6 @@ bgp_finish_attrs(struct bgp_parse_state *s, rta *a)
 	       p->cf->local_role == BGP_ROLE_RS_CLIENT))
       bgp_set_attr_u32(&a->eattrs, s->pool, BA_ONLY_TO_CUSTOMER, 0, p->cf->remote_as);
   }
-
-  /* Apply MPLS policy for labeled SAFIs */
-  if (s->mpls && s->proto->p.mpls_channel)
-  {
-    struct mpls_channel *mc = (void *) s->proto->p.mpls_channel;
-    ea_set_attr_u32(&a->eattrs, s->pool, EA_MPLS_POLICY, 0, EAF_TYPE_INT, mc->label_policy);
-  }
 }
 
 
